<?php
/*
 * Copyright (C) 2022 Intergo Telecom Ltd <support@sms.to>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Joomla\Component\Smsto\Administrator\Model;

use JLoader;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Model\AdminModel;
use Smsto\Helper;

\defined('_JEXEC') or die;

class SynchronizeModel extends AdminModel
{

    /**
     * Get list from SMSto. Create if not exist
     * 
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @param integer $group_id
     * @return void
     */
    public function getList(int $group_id) {
        JLoader::registerNamespace('Smsto', JPATH_LIBRARIES . '/smsto');

        $modelUserGroups = $this->bootComponent('com_users')
            ->getMVCFactory()->createModel('Group', 'Administrator', ['ignore_request' => true]);
        $group = $modelUserGroups->getItem($group_id);
        $params = ComponentHelper::getParams('com_smsto');
        $api_key = $params->get('api_key');

        ob_start();
        Helper::callSmsto($api_key, 'get', getenv('VITE_SMSTO_APP_URL') . 'v1/people/lists', json_encode(['name' => 'J_' . $group->title]));
        $response = json_decode(ob_get_clean(), true);
        if ($response['total'] == 0) { // Create missing lists
            Helper::callSmsto($api_key, 'post', getenv('VITE_SMSTO_APP_URL') . 'v1/people/lists/create', json_encode(['name' => 'J_' . $group->title]));
            ob_start();
            Helper::callSmsto($api_key, 'get', getenv('VITE_SMSTO_APP_URL') . 'v1/people/lists', json_encode(['name' => 'J_' . $group->title]));
            $response = json_decode(ob_get_clean(), true);
        }

        return $response['data'][0];
    }

    /**
     * Sync user phones on SMSto
     * 
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @param int $group_id
     * @return array
     */
    public function syncUsers(int $group_id)
    {
        $params = ComponentHelper::getParams('com_smsto');
        $api_key = $params->get('api_key');
        $list = $this->getList($group_id);
        $modelUserGroups = $this->bootComponent('com_users')
            ->getMVCFactory()->createModel('Group', 'Administrator', ['ignore_request' => true]);
        $group = $modelUserGroups->getItem($group_id);

        $db    = $this->getDbo();
        $query = $db->getQuery(true);
        $query->select([
            $db->quoteName('a.profile_value') . 'AS' . $db->quote('phone'),
            $db->quoteName('b.name') . 'AS' . $db->quote('firstname'),
            $db->quoteName('b.email') . 'AS' . $db->quote('email'),
        ])
            ->where($db->quoteName('a.profile_key') . '=' . $db->quote('profile.phone'))
            ->where($db->quoteName('c.group_id') . '=' . $db->quote($group->id))
            ->from($db->quoteName('#__user_profiles') . ' AS a')
            ->join('LEFT', '#__users AS b ON b.id = a.user_id')
            ->join('LEFT', '#__user_usergroup_map AS c ON c.user_id = a.user_id');

        $db->setQuery($query);
        $contactsList = $db->loadAssocList();
        $contacts = [];
        foreach ($contactsList as $key => $value) {
            $contacts[$key]["phone"] = trim($contactsList[$key]["phone"], "\"");
            $contacts[$key]["firstname"] = trim($contactsList[$key]["firstname"], "\"");
            $contacts[$key]["lastname"] = "";
            $contacts[$key]["email"] = trim($contactsList[$key]["email"], "\"");
        }
        if ($contacts) {
            $payload = [
                "user_lists" => json_encode([
                    0 => [
                        "id" => $list["id"],
                        "name" => $list["name"]
                    ]
                ]),
                "upload_method" => "true",
                "duplicate_strategy" => "update",
                "contacts" => json_encode($contacts),
                "parse_settings" => json_encode([
                    "delimiter" => ",",
                    "newline" => "",
                    "quoteChar" => "\"",
                    "escapeChar" => "\"",
                    "header" => false,
                    "dynamicTyping" => false,
                    "preview" => 0,
                    "encoding" => "",
                    "worker" => false,
                    "comments" => false,
                    "download" => false,
                    "skipEmptyLines" => false,
                    "transform" => false,
                    "delimitersToGuess" => [
                        0 => ",",
                        1 => "\t",
                        2 => "|",
                        3 => ";",
                        4 => "\x1E",
                        5 => "\x1F",
                    ]
                ]),
            ];

            ob_start();
            Helper::callSmsto($api_key, 'post', getenv('VITE_SMSTO_APP_URL') . 'v1/people/contacts/import', json_encode($payload));
            $response = json_decode(ob_get_clean(), true);
            return $response;
        }

        return [
            "success" => false,
            "message" => Text::_('COM_SMSTO_SYNC_STARTED')
        ];
    }

    /**
     * Method to get the record form.
     * 
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @param   array    $data      An optional array of data for the form to interrogate.
     * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
     *
     * @return  Form|bool  A Form object on success, false on failure
     *
     */
    public function getForm($data = array(), $loadData = false)
    {
        // Get the form.
        $form = $this->loadForm('com_smsto.synchronize', 'synchronize', array('control' => 'jform', 'load_data' => $loadData));

        if (empty($form)) {
            return false;
        }

        return $form;
    }

    /**
     * Get user smsto plugin
     * 
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @return int
     */
    public function getUserPlugin() {
        $db    = $this->getDbo();
        $query = $db->getQuery(true);
        $query->select([
            $db->quoteName('a.extension_id'),
            $db->quoteName('a.enabled'),
        ])
        ->where($db->quoteName('a.type') . '=' . $db->quote('plugin'))
        ->where($db->quoteName('a.element') . '=' . $db->quote('smsto'))
        ->where($db->quoteName('a.folder') . '=' . $db->quote('user'))
        ->from($db->quoteName('#__extensions') . ' AS a');

        $db->setQuery($query);
        return $db->loadAssoc();
    }
}
