<?php

/**
 * @package     Joomla.Site
 * @subpackage  com_smsto
 *
 * @copyright   (C) 2010 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Joomla\Component\Smsto\Site\Model;

use JLoader;
use Joomla\CMS\Factory;
use Joomla\CMS\Form\Form;
use Joomla\Http\Http;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Mail\MailTemplate;
use Joomla\CMS\MVC\Model\FormModel;
use Joomla\CMS\Router\Route;
use Joomla\CMS\String\PunycodeHelper;
use Joomla\CMS\Table\User;
use Joomla\Database\ParameterType;
use Joomla\Utilities\ArrayHelper;
use Smsto\Helper;
use stdClass;

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * Remind model class for Users.
 *
 * @since  1.5
 */
class RemindModel extends FormModel
{
    /**
     * Method to get the username remind request form.
     *
     * @param   array    $data      An optional array of data for the form to interrogate.
     * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
     *
     * @return  Form|bool A Form object on success, false on failure
     *
     * @since   1.6
     */
    public function getForm($data = array(), $loadData = true)
    {
        // Get the form.
        $form = $this->loadForm('com_smsto.remind', 'remind', array('control' => 'jform', 'load_data' => $loadData));

        if (empty($form)) {
            return false;
        }

        return $form;
    }

    /**
     * Override preprocessForm to load the user plugin group instead of content.
     *
     * @param   Form    $form   A Form object.
     * @param   mixed   $data   The data expected for the form.
     * @param   string  $group  The name of the plugin group to import (defaults to "content").
     *
     * @return  void
     *
     * @throws  \Exception if there is an error in the form event.
     *
     * @since   1.6
     */
    protected function preprocessForm(Form $form, $data, $group = 'user')
    {
        parent::preprocessForm($form, $data, 'user');
    }

    /**
     * Method to auto-populate the model state.
     *
     * Note. Calling getState in this method will result in recursion.
     *
     * @return  void
     *
     * @since   1.6
     *
     * @throws  \Exception
     */
    protected function populateState()
    {
        // Get the application object.
        $app = Factory::getApplication();
        $params = $app->getParams('com_smsto');

        // Load the parameters.
        $this->setState('params', $params);
    }

    /**
     * Send the remind username email
     *
     * @param   array  $data  Array with the data received from the form
     *
     * @return  boolean
     *
     * @since   1.6
     */
    public function processRemindRequest($data)
    {
        // load users language
        $app = Factory::getApplication();
        $app->getLanguage()->load('com_users', JPATH_SITE, 'en-GB', true);

        // Get the form.
        $form = $this->getForm();
        $data['email'] = PunycodeHelper::emailToPunycode($data['email']);

        // Check for an error.
        if (empty($form)) {
            return false;
        }

        // Validate the data.
        $data = $this->validate($form, $data);

        // Check for an error.
        if ($data instanceof \Exception) {
            return false;
        }

        // Check the validation results.
        if ($data === false) {
            // Get the validation messages from the form.
            foreach ($form->getErrors() as $formError) {
                $this->setError($formError->getMessage());
            }

            return false;
        }

        if (empty($data['phone']) && empty($data['email'])) {
            $this->setError(Text::sprintf('COM_SMSTO_EMPTY_PHONE_EMAIL'));
            return false;
        }

        if (!empty($data['phone'])) {
            $result = $this->sms($data);
            if ($result == false) {
                return $result;
            }
        }
        
        if (!empty($data['email'])) {
            $result = $this->email($data);
            if ($result == false) {
                return $result;
            }
        }
       
        return true;
    }

    /**
     * Get user from phone
     * 
     * @param array $data
     * @return stdClass
     */
    protected function sms(array $data)
    {
        $db    = $this->getDatabase();
        $query = $db->getQuery(true)
            ->select(
                [
                    $db->quoteName('user_id'),
                ]
            )
            ->from($db->quoteName('#__user_profiles'))
            ->where($db->quoteName('profile_key') . ' = ' . $db->quote('profile.phone'))
            ->where($db->quoteName('profile_value') . ' LIKE ' . $db->quote('%' . $data['phone'] . '%'));

        $db->setQuery($query);
        try {
            $user_id = $db->loadResult();
        } catch (\RuntimeException $e) {
            $this->setError(Text::sprintf('COM_USERS_DATABASE_ERROR', $e->getMessage()));
            return false;
        }
        if ($user_id === null) {
            $this->setError(Text::sprintf('COM_SMSTO_PHONE_NOT_FOUND', $data['phone']));
            return false;
        }

        $query = $db->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__users'))
            ->where($db->quoteName('id') . ' = ' . $db->quote($user_id));

        // Get the user id.
        $db->setQuery($query);

        try {
            $user = $db->loadObject();
        } catch (\RuntimeException $e) {
            $this->setError(Text::sprintf('COM_USERS_DATABASE_ERROR', $e->getMessage()));
            return false;
        }

        // Check for a user.
        if (empty($user)) {
            $this->setError(Text::_('COM_USERS_USER_NOT_FOUND'));

            return false;
        }
        
        // Make sure the user isn't blocked.
        if ($user->block) {
            $this->setError(Text::_('COM_USERS_USER_BLOCKED'));

            return false;
        }

        // Assemble the login link.
        $app = Factory::getApplication();
        $link = 'index.php?option=com_users&view=login';
        $mode = $app->get('force_ssl', 0) == 2 ? 1 : (-1);

        // Put together the email template payload.
        $payload = ArrayHelper::fromObject($user);
        $payload['sitename'] = $app->get('sitename');
        $payload['link_text'] = Route::_($link, false, $mode);
        $payload['link_html'] = Route::_($link, true, $mode);

        // Send message
        $message= Text::plural('COM_SMSTO_REMINDER_MESSAGE', $payload['sitename'], $payload['username'], $payload['link_text']);
        $params = $this->getState('params');
		JLoader::registerNamespace('Smsto', JPATH_LIBRARIES . '/smsto');
        Helper::send($params->get('api_key'), $data['phone'], $message);

        return true;
    }

    /**
     * Get user from email
     * 
     * @param array $data
     * @return stdClass
     */
    protected function email(array $data)
    {        
        // Find the user id for the given email address.
        $db = $this->getDatabase();
        $query = $db->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__users'))
            ->where('LOWER(' . $db->quoteName('email') . ') = LOWER(:email)')
            ->bind(':email', $data['email']);

        // Get the user id.
        $db->setQuery($query);

        try {
            $user = $db->loadObject();
        } catch (\RuntimeException $e) {
            $this->setError(Text::sprintf('COM_USERS_DATABASE_ERROR', $e->getMessage()));
            return false;
        }

        // Check for a user.
        if (empty($user)) {
            $this->setError(Text::_('COM_USERS_USER_NOT_FOUND'));

            return false;
        }

        // Make sure the user isn't blocked.
        if ($user->block) {
            $this->setError(Text::_('COM_USERS_USER_BLOCKED'));

            return false;
        }

        // Assemble the login link.
        $app = Factory::getApplication();
        $link = 'index.php?option=com_users&view=login';
        $mode = $app->get('force_ssl', 0) == 2 ? 1 : (-1);

        // Put together the email template data.
        $data = ArrayHelper::fromObject($user);
        $data['sitename'] = $app->get('sitename');
        $data['link_text'] = Route::_($link, false, $mode);
        $data['link_html'] = Route::_($link, true, $mode);

        $mailer = new MailTemplate('com_users.reminder', $app->getLanguage()->getTag());
        $mailer->addTemplateData($data);
        $mailer->addRecipient($user->email, $user->name);

        // Try to send the password reset request email.
        try {
            $return = $mailer->send();
        } catch (\Exception $exception) {
            try {
                Log::add(Text::_($exception->getMessage()), Log::WARNING, 'jerror');

                $return = false;
            } catch (\RuntimeException $exception) {
                Factory::getApplication()->enqueueMessage(Text::_($exception->errorMessage()), 'warning');

                $return = false;
            }
        }

        // Check for an error.
        if ($return !== true) {
            $this->setError(Text::_('COM_USERS_MAIL_FAILED'));

            return false;
        }

        Factory::getApplication()->triggerEvent('onUserAfterRemind', array($user));

        return true;
    }

}
