<?php
/*
 * Copyright (C) 2022 Intergo Telecom Ltd <support@sms.to>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

defined('_JEXEC') or die;

use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\Plugin\User\Smsto\Traits\UserLogin;
use Joomla\Plugin\User\Smsto\Traits\UserLoginFail;
use Joomla\Plugin\User\Smsto\Traits\UserLogout;
use Joomla\Plugin\User\Smsto\Traits\UserRegister;
use Smsto\Helper;

/**
 * Joomla User plugin
 *
 * @since  1.5
 */
class PlgUserSmsto extends CMSPlugin
{
	use UserLogin, UserLogout, UserLoginFail, UserRegister;

	/**
	 * @var    \Joomla\CMS\Application\CMSApplication
	 *
	 * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
	 */
	protected $app;

	/**
	 * @var    \Joomla\Database\DatabaseDriver
	 * 
	 * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
	 *
	 */
	protected $db;

	/**
	 * com_smsto component
	 *
	 * @var \Joomla\Component\Smsto\Administrator\Extension\SmstoComponent
	 */
	protected $component;

	/**
	 * Remove all sessions for the user name
	 *
	 * Method is called after user data is deleted from the database
	 * 
	 * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
	 *
	 * @param   array    $user     Holds the user data
	 * @param   boolean  $success  True if user was successfully stored in the database
	 * @param   string   $msg      Message
	 *
	 * @return  void
	 *
	 */
	public function onUserAfterDelete($user, $success, $msg): void
	{
		if (!$success) {
			return;
		}

		$this->loadLanguage();
		JLoader::registerNamespace('Smsto', JPATH_LIBRARIES . '/smsto');
		$this->component = Factory::getApplication()->bootComponent('com_smsto');

		if ($this->params->get('sync_phones', 1)) {
			$contact = $this->deleteContact($user);
		}
	}

	/**
	 * Utility method to act on a user after it has been saved.
	 *
	 * This method sends a registration email to new users created in the backend.
	 * 
	 * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
	 *
	 * @param   array    $user     Holds the new user data.
	 * @param   boolean  $isnew    True if a new user is stored.
	 * @param   boolean  $success  True if user was successfully stored in the database.
	 * @param   string   $msg      Message.
	 *
	 * @return  void
	 *
	 */
	public function onUserAfterSave($user, $isnew, $success, $msg): void
	{
		if (!$success) {
			return;
		}

		$this->loadLanguage();
		JLoader::registerNamespace('Smsto', JPATH_LIBRARIES . '/smsto');
		$this->component = Factory::getApplication()->bootComponent('com_smsto');

		if ($this->params->get('sync_phones', 1)) {
			$this->createContact($user);
		}

		if ($this->params->get('notify_admin_user_registration', 1) && $isnew) {
			$this->notifyAdminRegister($user);
		}
	}

	/**
	 *
	 * Update user phone to smsto
	 * 
	 * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
	 *
	 * @param   array    $user     Holds the new user data.
	 *
	 * @return  void
	 *
	 */
	protected function createContact(array $user): void
	{
		if (!isset($user['profile']['phone'])) {
			return;
		}

		$list_ids = [];
		$modelSynchronize = $this->component->getMVCFactory()->createModel('Synchronize', 'Administrator', ['ignore_request' => true]);
		foreach ($user['groups'] as $group_id) {
			$list = $modelSynchronize->getList($group_id);
			$list_ids[] = $list['id'];
		}
		$payload = [
			'phone' => $user['profile']['phone'],
			'firstname' => $user['name'],
			'email' => $user['email'],
			'list_ids' => $list_ids
		];

		$params = ComponentHelper::getParams('com_smsto');
		$api_key = $params->get('api_key');
		ob_start();
		Helper::callSmsto($api_key, 'post', getenv('VITE_SMSTO_APP_URL') . 'v1/people/contacts/create', json_encode($payload));
		$response = json_decode(ob_get_clean(), true);
	}

	/**
	 * Get a contact from it's phone
	 * 
	 * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
	 *
	 * @param array $user
	 * @return array|bool
	 */
	protected function getContact(array $user): array|bool
	{
		$query = $this->db->getQuery(true);
		$query->select([
			$this->db->quoteName('a.profile_value') . 'AS' . $this->db->quote('phone'),
		])
			->from($this->db->quoteName('#__user_profiles', 'a'))
			->where($this->db->quoteName('a.user_id') . ' = ' . $user['id'])
			->where($this->db->quoteName('a.profile_key') . '=' . $this->db->quote('profile.phone'));
		$this->db->setQuery($query);
		$phone = trim($this->db->loadResult(), '"');

		$payload = [
			'search' => $phone,
		];
		$params = ComponentHelper::getParams('com_smsto');
		$api_key = $params->get('api_key');
		ob_start();
		Helper::callSmsto($api_key, 'get', getenv('VITE_SMSTO_APP_URL') . 'v1/people/contacts', json_encode($payload));
		$response = json_decode(ob_get_clean(), true);
		if ($response['total'] > 0) {
			return $response['data'][0];
		}
		return false;
	}

	/**
	 * Delete a contact from SMSto
	 * 
	 * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
	 *
	 * @param array $user
	 * @return void
	 */
	protected function deleteContact(array $user): void
	{
		$contact = $this->getContact($user);
		$params = ComponentHelper::getParams('com_smsto');
		$api_key = $params->get('api_key');
		ob_start();
		Helper::callSmsto($api_key, 'delete', getenv('VITE_SMSTO_APP_URL') . "v1/people/contacts/" . $contact['id'] . "/delete", []);
		$response = json_decode(ob_get_clean(), true);
	}

	/**
	 * Get phones from super administrators that have phones
	 * 
	 * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
	 *
	 * @return array
	 */
	protected function getAdministratorsPhones() {
		$query = $this->db->getQuery(true);
		$query->select([
			$this->db->quoteName('a.profile_value') . 'AS' . $this->db->quote('phone'),
		])
			->where($this->db->quoteName('a.profile_key') . '=' . $this->db->quote('profile.phone'))
			->from($this->db->quoteName('#__user_profiles') . ' AS a')
			->join('LEFT', '#__users AS b ON b.id = a.user_id')
			->where($this->db->quoteName('b.sendEmail') . ' = 1')
			->where($this->db->quoteName('b.block') . ' = 0');

		$this->db->setQuery($query);
		$phones = $this->db->loadRow();
		foreach ($phones as $key => $phone) {
			$phones[$key] = trim($phone, '"');
		}
		return $phones;
	}

}
